package frc.robot.subsystems;

import com.revrobotics.CANSparkMax;
import com.revrobotics.RelativeEncoder;
import com.revrobotics.CANSparkLowLevel.MotorType;

import edu.wpi.first.math.controller.ArmFeedforward;
import edu.wpi.first.math.controller.PIDController;
import edu.wpi.first.math.util.Units;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.constants.IntakePivotConstants;

public class IntakePivot extends SubsystemBase {
    private CANSparkMax pivotMotor;

    private RelativeEncoder intakeEncoder;

    private PIDController intakePIDController;
    private ArmFeedforward intakeFF;

    public IntakePivot() {
        pivotMotor = new CANSparkMax(0, MotorType.kBrushless);

        intakeEncoder = pivotMotor.getEncoder();

        intakeFF = new ArmFeedforward(
            IntakePivotConstants.kFFS, 
            IntakePivotConstants.kFFG, 
            IntakePivotConstants.kFFV
        );

        intakePIDController = new PIDController(
            IntakePivotConstants.kPPivot, 
            IntakePivotConstants.kIPivot, 
            IntakePivotConstants.kDPivot
        );
    }

    public Command intakeUp() {
        return run(() -> {
            pivotMotor.setVoltage(
                intakePIDController.calculate(
                    getIntakeAngle(), 
                    IntakePivotConstants.kUpAngle
                ) + intakeFF.calculate(
                    IntakePivotConstants.kUpAngle, 
                    0.0
                )
            );
        });
    }

    public Command intakeDown() {
        return run(() -> {
            pivotMotor.setVoltage(
                intakePIDController.calculate(
                    getIntakeAngle(), 
                    IntakePivotConstants.kDownAngle
                ) + intakeFF.calculate(
                    IntakePivotConstants.kDownAngle, 
                    0.0
                )
            );
        });
    }

    public double getIntakeAngle(){
        return Units.rotationsToRadians(
            intakeEncoder.getPosition()/
            IntakePivotConstants.kIntakePivotConversionFactor
        );
    }
}